#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
import re
import aiohttp
import asyncio

from ..lib import cfg, logger

LOGGER = logger.get('消息通知')


async def wecom_app(title: str, content: str) -> None:
    """
    通过 企业微信 APP 推送消息。
    """
    if not cfg.get("QYWX_AM"):
        LOGGER.error("QYWX_AM 未设置!!\n取消推送")
        return
    QYWX_AM_AY = re.split(",", cfg.get("QYWX_AM"))
    if 4 < len(QYWX_AM_AY) > 5:
        LOGGER.error("QYWX_AM 设置错误!!\n取消推送")
        return
    LOGGER.info("企业微信 APP 服务启动")

    corpid = QYWX_AM_AY[0]
    corpsecret = QYWX_AM_AY[1]
    touser = QYWX_AM_AY[2]
    agentid = QYWX_AM_AY[3]
    try:
        media_id = QYWX_AM_AY[4]
    except IndexError:
        media_id = ""
    wx = WeCom(corpid, corpsecret, agentid)
    # 如果没有配置 media_id 默认就以 text 方式发送
    if not media_id:
        message = title + "\n\n" + content
        response = await wx.send_text(message, touser)
    else:
        response = await wx.send_mpnews(title, content, media_id, touser)

    if response == "ok":
        LOGGER.info("企业微信推送成功！")
    else:
        LOGGER.info("企业微信推送失败！错误信息如下：\n", response)


class WeCom:
    def __init__(self, corpid, corpsecret, agentid):
        self.CORPID = corpid
        self.CORPSECRET = corpsecret
        self.AGENTID = agentid
        self.ORIGIN = "https://qyapi.weixin.qq.com"
        if cfg.get("QYWX_ORIGIN"):
            self.ORIGIN = cfg.get("QYWX_ORIGIN")

    async def get_access_token(self):
        url = f"{self.ORIGIN}/cgi-bin/gettoken"
        values = {
            "corpid": self.CORPID,
            "corpsecret": self.CORPSECRET,
        }
        async with aiohttp.ClientSession() as session:
            async with session.post(url, params=values) as resp:
                data = await resp.json()
                return data["access_token"]

    async def send_text(self, message, touser="@all"):
        access_token = await self.get_access_token()
        send_url = f"{self.ORIGIN}/cgi-bin/message/send?access_token={access_token}"
        send_values = {
            "touser": touser,
            "msgtype": "text",
            "agentid": self.AGENTID,
            "text": {"content": message},
            "safe": "0",
        }
        async with aiohttp.ClientSession() as session:
            async with session.post(send_url, json=send_values) as resp:
                response = await resp.json()
                return response["errmsg"]

    async def send_mpnews(self, title, message, media_id, touser="@all"):
        access_token = await self.get_access_token()
        send_url = f"{self.ORIGIN}/cgi-bin/message/send?access_token={access_token}"
        send_values = {
            "touser": touser,
            "msgtype": "mpnews",
            "agentid": self.AGENTID,
            "mpnews": {
                "articles": [
                    {
                        "title": title,
                        "thumb_media_id": media_id,
                        "author": "Author",
                        "content_source_url": "",
                        "content": message.replace("\n", "<br/>"),
                        "digest": message,
                    }
                ]
            },
        }
        async with aiohttp.ClientSession() as session:
            async with session.post(send_url, json=send_values) as resp:
                response = await resp.json()
                return response["errmsg"]


async def wecom_bot(title: str, content: str, mentioned_list: list = None, mentioned_mobile_list: list = None, api_key: str = None) -> None:
    """
    通过 企业微信机器人 群消息推送消息。

    Args:
        :param title: 消息标题
        :param content: 消息内容
        :param mentioned_list: 需要@的用户ID列表（userid的列表，@all表示提醒所有人）
        :param mentioned_mobile_list: 需要@的手机号列表（手机号列表，@all表示提醒所有人）
        :param api_key: 接口token，为空时从环境变量中获取
    """
    _key = api_key or cfg.get("QYWX_KEY")
    if not _key:
        LOGGER.error("企业微信机器人 服务的 QYWX_KEY 未设置!!\n取消推送")
        return
    LOGGER.info("企业微信机器人服务启动")

    origin = "https://qyapi.weixin.qq.com"
    if cfg.get("QYWX_ORIGIN"):
        origin = cfg.get("QYWX_ORIGIN")

    url = f"{origin}/cgi-bin/webhook/send?key={_key}"
    headers = {"Content-Type": "application/json;charset=utf-8"}

    # 构建消息数据
    data = {
        "msgtype": "text",
        "text": {
            "content": f"{title}\n\n{content}"
        }
    }

    # 添加@功能
    if mentioned_list or mentioned_mobile_list:
        if mentioned_list:
            data["text"]["mentioned_list"] = mentioned_list
        if mentioned_mobile_list:
            data["text"]["mentioned_mobile_list"] = mentioned_mobile_list

    async with aiohttp.ClientSession() as session:
        try:
            async with session.post(
                    url=url, json=data, headers=headers, timeout=15
            ) as resp:
                response = await resp.json()
                if response["errcode"] == 0:
                    LOGGER.info("企业微信机器人推送成功！")
                else:
                    LOGGER.error(f"企业微信机器人推送失败！错误码: {response['errcode']}, 错误信息: {response['errmsg']}")
        except asyncio.TimeoutError:
            LOGGER.error("企业微信机器人推送超时！")
        except Exception as e:
            LOGGER.error(f"企业微信机器人推送出错：{e}")



# -*- coding: UTF-8 -*-
"""常用工具类"""
import random
import uuid
from datetime import datetime


# 生成批次编号
def gen_batch_id():
    # 时间(3位毫秒)+随机数(3位)
    # return f"{datetime.now().strftime('%y%m%d%H%M%S%f')[0:10]}{random.randint(1000, 9999)}"
    return f"{datetime.now().strftime('%y%m%d%H%M%S')}{random.randint(1000, 9999)}"


# 生成消息编号
def gen_msg_id():
    # 时间(3位毫秒)+随机数(3位)
    # return f"{datetime.now().strftime('%y%m%d%H%M%S%f')[0:10]}{random.randint(1000, 9999)}"
    # return f"{datetime.now().strftime('%y%m%d%H%M%S%f')}{random.randint(1000, 9999)}"
    return uuid.uuid4().hex


def gen_random_code(n=8, alpha=True):
    """
    生成验证码
    :param n:
    :param alpha:
    :return:
    """
    s = ''  # 创建字符串变量,存储生成的验证码
    for i in range(n):  # 通过for循环控制验证码位数
        num = random.randint(0, 9)  # 生成随机数字0-9
        if alpha:  # 需要字母验证码,不用传参,如果不需要字母的,关键字alpha=False
            upper_alpha = chr(random.randint(65, 90))
            lower_alpha = chr(random.randint(97, 122))
            num = random.choice([num, upper_alpha, lower_alpha])
        s = s + str(num)
    return s


# 测试
if __name__ == '__main__':
    while True:
        print(gen_msg_id())



# encoding: utf-8
# version: redis==5.0.7

from ..lib import cfg, logger
from ..exception.exceptions import NoConfigException

LOGGER = logger.get("AsyncRedisClient")


class AsyncRedis(object):

    def __init__(self, redis_uri=None, redis_db=None, decode_responses=True):
        # 按需导入（延迟加载）
        from redis import asyncio as aioredis

        self.redis_uri = redis_uri
        self.redis_db = redis_db
        if not self.redis_uri:
            self.redis_uri = cfg.get("REDIS_URL")
        if not self.redis_db:
            self.redis_db = cfg.get("REDIS_DB")

        if not self.redis_uri:
            raise NoConfigException("redis_uri not config!")
        if not self.redis_db:
            raise NoConfigException("redis_db not config!")

        self.client = aioredis.Redis.from_url(url=self.redis_uri, db=self.redis_db, decode_responses=decode_responses)
        LOGGER.info(f'init async redis client: uri={self.redis_uri}, db={self.redis_db}')

    def __str__(self):
        return f"AsyncRedis client at: {self.redis_uri}/{self.redis_db}"

    async def close(self):
        if self.client:
            try:
                await self.client.aclose()
            except Exception as e:
                print(e)



import io
import os
import sys
import json
import getopt
from logging.config import dictConfig
from typing import Dict, Any
from dotenv.main import DotEnv, load_dotenv
from contextlib import contextmanager

from .lib import logger


def _fix_dotenv():
    def my_get_stream(self):
        """重写python-dotenv读取文件的方法，使用utf-8，支持读取中文"""
        if isinstance(self.dotenv_path, io.StringIO):
            yield self.dotenv_path
        elif os.path.isfile(self.dotenv_path):
            with io.open(self.dotenv_path, encoding='utf-8') as stream:
                yield stream
        else:
            if self.verbose:
                print("File doesn't exist %s", self.dotenv_path)
            yield io.StringIO('')

    DotEnv._get_stream = contextmanager(my_get_stream)


def get_cmd_opts() -> Dict[str, Any]:
    """
    get commandline opts
    :return: cmd options
    """
    # get options
    # -a app / -e env / -t tag
    try:
        opts, _ = getopt.getopt(
            sys.argv[1:],
            'a:s:f:e:t:',
            ['app=', 'script=', 'func=', 'env=', 'tag=']
        )
    except getopt.GetoptError as e:
        raise e
    t = {
        'app': '',
        'script': '',
        'func': 'start',
        'env': 'dev',
        'tag': ''
    }
    for o, a in opts:
        if o == '-e':
            t['env'] = a
        elif o == '-f':
            t['func'] = a
        elif o == '-s':
            t['script'] = a
        elif o == '-a':
            t['app'] = a
        elif o == '-t':
            t['tag'] = a
    return t


def load_cfg(app: str, script: str, func: str, env: str):
    """
    load configs
    :param func:
    :param script:
    :param app:
    :param env:
    :return:
    """
    if not env:
        raise Exception('env not specified')
    if not func:
        raise Exception('func not specified')
    if not script:
        raise Exception('script not specified')
    if not app:
        raise Exception('app not specified')
    script_file = script
    if script.endswith('.py'):
        script = script[:-3]
    if not script_file.endswith('.py'):
        script_file = f'{script}.py'
    assert os.path.exists(os.path.join(os.getcwd(), app, script_file))
    cfg_dir = os.path.join(app, 'cfg', env)
    assert os.path.isdir(cfg_dir)

    # env file
    env_file = os.path.join(cfg_dir, 'app.cfg')
    print(f"Loading env file from: {env_file}")
    load_dotenv(dotenv_path=env_file)

    # logger cfg
    logger_cfg_file = os.path.join(cfg_dir, 'logger.json')
    print(f"Loading logger config from: {logger_cfg_file}")
    logger_cfg = json.loads(open(logger_cfg_file, encoding='utf-8').read())
    assert isinstance(logger_cfg, dict)
    dictConfig(logger_cfg)

    return {
        'env': env,
        'app': app,
        'script': script,
        'script_file': script_file,
        'func': func,
        'log_config': logger_cfg,
        'env_file': env_file
    }


def init_config() -> Dict:
    _fix_dotenv()
    opts = get_cmd_opts()
    cfg = load_cfg(opts['app'], opts['script'], opts['func'], opts['env'])

    LOGGER = logger.get()
    LOGGER.info('================')
    LOGGER.info(cfg)
    LOGGER.info('================')

    return cfg



# -*- coding: utf-8 -*-
class BizException(Exception):
    def __init__(self, message: str, code: int = -1):
        self.message = message
        self.code = code


class NoAuthException(Exception):
    def __init__(self, message: str = '权限不足'):
        self.message = message
        self.code = 401


# 系统错误类异常
class SystemException(Exception):
    def __init__(self, message: str = '系统异常，请联系管理员处理'):
        self.message = message
        self.code = 500


# 系统参数为配置
class NoConfigException(SystemException):
    def __init__(self, message: str = '系统参数配置异常'):
        self.message = message
        self.code = 500


class MQConnectionException(SystemException):
    """RabbitMQ连接异常"""

    def __init__(self, message="MQ connection failed"):
        super().__init__(message)


# -----------
# 采集专用
# -----------

# 需要重试的请求统一包装
class RetryRequest(Exception):
    def __init__(self, ex=None):
        self.ex = ex


# 后端专用异常类型，信息不直接暴露给前端
class BackendException(Exception):
    def __init__(self, message: str = '服务端异常'):
        self.message = message


# 代理获取失败异常
class ProxyGetFailException(BackendException):
    def __init__(self, message: str = '代理加载失败'):
        self.message = message


# 会话创建失败异常
class SessionInitException(BackendException):
    def __init__(self, message: str = '会话创建失败异常'):
        self.message = message


# 接口失败异常
class ApiFailException(BackendException):
    def __init__(self, message: str = '接口调用失败'):
        self.message = message


class OfflineException(Exception):
    """商品下架异常"""
    meta = None

    def __init__(self, meta=None):
        self.meta = meta


class NotExistException(Exception):
    """商品不存在异常"""
    meta = None

    def __init__(self, meta=None):
        self.meta = meta


class NotStockException(Exception):
    """商品无库存异常"""
    meta = None

    def __init__(self, meta=None):
        self.meta = meta


class NotKnownException(Exception):
    """商品未知异常"""
    meta = None

    def __init__(self, meta=None):
        self.meta = meta


class AbandonException(Exception):
    """放弃异常"""
    meta = None

    def __init__(self, meta=None):
        self.meta = meta



# encoding: utf-8
# version: pymongo==4.7.3, motor==3.5.0
from ..exception.exceptions import BizException
from ..exception.exceptions import NoConfigException
from ..lib import cfg, logger

LOGGER = logger.get('AsyncMongoClient')


class AsyncMongo(object):
    def __init__(self, mongo_uri: str = None, mongo_db: str = None):
        # 按需导入（延迟加载）
        import motor.motor_asyncio

        self.mongo_uri = mongo_uri or cfg.get('MONGO_URL')
        self.mongo_db = mongo_db or cfg.get('MONGO_DB')
        if not self.mongo_uri:
            raise NoConfigException('mongodb uri not config!')
        if not self.mongo_db:
            raise NoConfigException('mongodb database not config!')

        self.client = motor.motor_asyncio.AsyncIOMotorClient(self.mongo_uri)
        self.db = self.client[self.mongo_db]

        LOGGER.info(self.client)

    # 获取集合
    def get_collection(self, coll):
        # 非异步
        return self.db.get_collection(coll)

    # 查询对象
    async def get(self, collection, query={}):
        return await self.db[collection].find_one(query)

    # 统计数量
    async def count(self, collection, query={}):
        return await self.db[collection].count_documents(query)

    # 查询列表
    async def list(self, collection, query={}, fields=None, sort=[], batch_size: int = 2000):
        result = []
        cursor = self.db[collection].find(filter=query, projection=fields, sort=sort)
        cursor.batch_size(batch_size)
        async for document in cursor:
            result.append(document)
        await cursor.close()

        return result

    async def list_with_cursor(self, collection, query={}, fields=None, sort=[], batch_size: int = 2000):
        """
        样例:
            cursor = await aio_mongo.list_with_cursor(collection)
            async for document in cursor:
                print("document =", document)
            await cursor.close()
        注意：应该在使用完毕后主动关闭 cursor
        """
        cursor = self.db[collection].find(filter=query, projection=fields, sort=sort)
        cursor.batch_size(batch_size)

        return cursor

    # 分页查询
    async def page(self, collection, query={}, page_no=1, page_size=20, fields=None, sort=[], batch_size: int = 2000):
        total = await self.db[collection].count_documents(query) or 0
        cursor = self.db[collection].find(query, fields, sort=sort).skip(page_size * (page_no - 1)).limit(page_size)
        cursor.batch_size(batch_size)
        rows = []
        async for document in cursor:
            rows.append(document)
        await cursor.close()

        return rows, total

    # 查询列表前N个
    async def top(self, collection, query={}, sort=[], limit=1, fields=None, batch_size: int = 2000):
        cursor = self.db[collection].find(filter=query, projection=fields, sort=sort, limit=limit)
        cursor.batch_size(batch_size)
        rows = []
        async for document in cursor:
            if limit == 1:
                return document
            rows.append(document)
        await cursor.close()

        return rows

    # 查询去重列表
    async def distinct(self, collection, dist_key, query={}, fields=None):
        return await self.db[collection].find(query, fields).distinct(dist_key)

    # 含分页聚合查询
    async def aggregate_page(self, collection, pipelines, page_no=1, page_size=20):
        skip = page_size * (page_no - 1)
        if pipelines:
            pipelines.append({'$facet': {'total': [{'$count': 'count'}], 'rows': [{'$skip': skip}, {'$limit': page_size}]}})
            pipelines.append({'$project': {'data': '$rows', 'total': {'$arrayElemAt': ['$total.count', 0]}}})

            cursor = self.db[collection].aggregate(pipelines, session=None, allowDiskUse=True)
            cursor.batch_size(page_size)
            async for rs in cursor:
                # rs: dict as {"data": [], "total": int}
                if rs and 'data' in rs and 'total' in rs:
                    await cursor.close()
                    return rs.get('data'), rs.get('total')
            await cursor.close()
        return [], 0

    # 聚合查询
    async def aggregate(self, collection, pipelines=[], batch_size: int = 2000):
        """
        样例:
            cursor = await aio_mongo.aggregate(collection="collection_name", pipelines=[])
            async for row in cursor:
                print(row)

            await cursor.close()
        注意：应该在使用完毕后主动关闭 cursor
        """
        cursor = self.db[collection].aggregate(pipelines, session=None, allowDiskUse=True)
        cursor.batch_size(batch_size)
        return cursor

    # 查询分页列表,还没找到实际应用
    async def list_with_page(self, collection, query={}, page_size=10000, fields=None):
        # 没有用到
        rows = list()
        total = await self.db[collection].count_documents(query)
        if total > 0 and page_size > 0:
            total_page = round(total / page_size) + 1
            for page in range(0, total_page):
                if fields:
                    cursor = self.db[collection].find(query, fields).skip(page_size * page).limit(page)
                else:
                    cursor = self.db[collection].find(query).skip(page_size * page).limit(page)
                async for document in cursor:
                    rows.append(document)
                await cursor.close()
        return rows

    # 插入或更新
    async def insert_or_update(self, collection, data, id_key='_id', update=None, upsert=True, multi=False):
        if not multi:
            if data and not update:
                result = await self.db[collection].update_one({id_key: data[id_key]}, {'$set': data}, upsert=upsert)
            elif not data and update:
                result = await self.db[collection].update_one({id_key: data[id_key]}, update, upsert=upsert)
            else:
                # all([data, update]) or not all([data, update])
                raise BizException("data和update不能同时存在或同时为空")
        else:
            if data and not update:
                result = await self.db[collection].update_many({id_key: data[id_key]}, {'$set': data}, upsert=upsert)
            elif not data and update:
                result = await self.db[collection].update_many({id_key: data[id_key]}, update, upsert=upsert)
            else:
                raise BizException("data和update不能同时存在或同时为空")

        return result

    # 插入
    async def insert(self, collection, data):
        return await self.db[collection].insert_one(data)

    # 更新
    async def update(self, collection, filter, data=None, update=None, multi=False):
        if multi:
            if data and not update:
                result = await self.db[collection].update_many(filter, {'$set': data})
            elif not data and update:
                result = await self.db[collection].update_many(filter, update)
            else:
                raise BizException("data和update不能同时存在或同时为空")
        else:
            if data and not update:
                result = await self.db[collection].update_one(filter, {'$set': data})
            elif not data and update:
                result = await self.db[collection].update_one(filter, update)
            else:
                raise BizException("data和update不能同时存在或同时为空")

        return result

    # 原生保存方法
    async def save(self, collection, filter: dict, save_data: dict, upsert=True):
        return await self.db[collection].update_one(filter, {'$set': save_data}, upsert=upsert)

    # 以主键更新
    async def update_by_pk(self, collection, pk_val, data=None, update=None, upsert=False):
        if data and not update:
            result = await self.db[collection].update_one({'_id': pk_val}, {'$set': data}, upsert=upsert)
        elif not data and update:
            result = await self.db[collection].update_one({'_id': pk_val}, update, upsert=upsert)
        else:
            raise BizException("data和update不能同时存在或同时为空")
        return result

    # 批量更新
    async def batch_update(self, collection, filter, datas=None, update=None, *args, **kwargs):
        if datas and not update:
            result = await self.db[collection].update_many(filter, {'$set': datas})
        elif not datas and update:
            result = await self.db[collection].update_many(filter, update)
        else:
            raise BizException("data和update不能同时存在或同时为空")
        return result

    # 删除
    async def delete(self, collection, filter):
        return await self.db[collection].delete_many(filter)

    # 插入或更新
    async def bulk_write(self, collection, bulk_list: list, batch_size: int = 1000):
        result = None
        if bulk_list:
            bulk_lists = [bulk_list[i: i + batch_size] for i in range(0, len(bulk_list), batch_size)]
            for _bulk_list in bulk_lists:
                result = await self.db[collection].bulk_write(_bulk_list, ordered=False, bypass_document_validation=True)
        return result

    # 创建索引
    async def create_index(self, collection, fields):
        return await self.db[collection].create_index(fields)

    def close(self):
        if self.client:
            try:
                self.client.close()
                print("close successful")
            except Exception as e:
                print("close mongo client catch err:", e)




# -*- coding: utf-8 -*-
from tenacity import retry, wait_exponential, stop_after_attempt
from ..exception.exceptions import NoConfigException
from ..lib import cfg, logger

LOGGER = logger.get('AsyncRabbitClient')


class AsyncRabbitClient:
    def __init__(self, connection_uri: str = None):
        self.connection_uri = connection_uri or cfg.get('RABBITMQ_URI')
        if not self.connection_uri:
            raise NoConfigException('RABBITMQ_URI not configured!')
        self.connection = None
        self.channel = None
        self.reconnect_max_retries = 5
        self.reconnect_wait_base = 2

    @retry(
        wait=wait_exponential(multiplier=1, max=10),
        stop=stop_after_attempt(5),
        reraise=True
    )
    async def connect(self):
        """建立带有自动重连的RabbitMQ连接"""
        LOGGER.info(f'Connecting to RabbitMQ: {self.connection_uri}')
        try:
            # 按需导入（延迟加载）
            import aio_pika

            self.connection = await aio_pika.connect_robust(
                self.connection_uri,
                timeout=10
            )
            self.channel = await self.connection.channel()
            await self.channel.set_qos(prefetch_count=10)
            LOGGER.info('RabbitMQ connection established')
        except Exception as e:
            LOGGER.error(f'Connection failed: {str(e)}')
            raise

    async def get_channel(self):
        """获取有效通道（带健康检查）"""
        if not self.connection or self.connection.is_closed:
            await self.reconnect()
        return self.channel

    async def reconnect(self):
        """自动重连机制"""
        LOGGER.warning('Attempting to reconnect...')
        print(f'---------------> reconnect.....')
        try:
            if self.connection and not self.connection.is_closed:
                await self.connection.close()
            await self.connect()
        except Exception as e:
            LOGGER.error(f'Reconnect failed: {str(e)}')
            raise

    async def close(self):
        """安全关闭连接"""
        if self.connection and not self.connection.is_closed:
            await self.connection.close()
            LOGGER.info('Connection closed gracefully')

    async def __aenter__(self):
        """异步上下文管理器入口"""
        await self.connect()
        return self

    async def __aexit__(self, exc_type, exc, tb):
        """异步上下文管理器出口"""
        await self.close()



# encoding: utf-8
import asyncio
import time
import ssl
import random
from typing import Any, Dict, Union
from curl_cffi.requests import BrowserType, RequestsError, AsyncSession

from . import proxy as proxy_client
from ..lib import logger
from ..exception.exceptions import ProxyGetFailException, RetryRequest

LOGGER = logger.get('CFFI请求')


class SSLFactory:
    ciphers = ('ECDH+AESGCM:DH+AESGCM:ECDH+AES256:DH+AES256:ECDH+AES128:DH+AES:ECDH+HIGH:'
               'DH+HIGH:ECDH+3DES:DH+3DES:RSA+AESGCM:RSA+AES:RSA+HIGH:RSA+3DES').split(":")

    @staticmethod
    def gen() -> ssl.SSLContext:
        random.shuffle(SSLFactory.ciphers)
        ciphers = ":".join(SSLFactory.ciphers)
        ciphers = ciphers + ":!aNULL:!eNULL:!MD5"
        context = ssl.create_default_context()
        context.set_ciphers(ciphers)
        return context


class CffiClient:

    def __init__(
            self,
            timeout: Union[float, tuple] = None,
            headers=None,
            ssl_context: ssl.SSLContext = None,
            allow_redirects: bool = True,
            enable_proxy: bool = True,
            proxy_protocol: str = 'http',
            session: AsyncSession = None
    ):
        self.timeout = timeout
        self.headers = headers
        self.ssl_context = ssl_context or SSLFactory.gen()
        self.allow_redirects = allow_redirects
        # 全局启用代理
        self.enable_proxy = enable_proxy

        # 代理
        self.proxy = None
        self.proxy_protocol = proxy_protocol or 'http'
        # 会话
        self.session = None
        self.ensure_session(session)

        self.enum_list = list()
        for tuple_item in BrowserType.__members__.items():
            self.enum_list.append(tuple_item[1])

    async def get(
            self, url, *,
            params: Any = None, timeout: Union[float, tuple] = None,
            cookies: Dict = None, headers: Dict = None, allow_redirects: bool = True,
            enable_proxy: Union[bool] = None, impersonate: str = None, verify: bool = True, proxy: Any = None
    ):
        try:
            # 会话
            self.ensure_session()
            # 代理配置
            if proxy:
                _proxy = proxy
            else:
                _enable_proxy = self.enable_proxy
                if enable_proxy is not None:
                    _enable_proxy = enable_proxy
                if _enable_proxy and not self.proxy:
                    await self.init_proxy(_enable_proxy)
                _proxy = self.proxy
                if not _enable_proxy:
                    _proxy = None

            start = time.time()
            _impersonate = random.choice(self.enum_list)
            return await self.session.get(
                url=url, params=params,
                headers=headers, cookies=cookies, proxy=_proxy,
                allow_redirects=allow_redirects, timeout=timeout, impersonate=impersonate, verify=verify
            )
        except (ProxyGetFailException, RequestsError) as tme:
            LOGGER.info(f"超时[{self.proxy}]: {type(tme)}")
            raise RetryRequest(ex=tme)
        except Exception as e:
            LOGGER.info(f"异常[{self.proxy}]: {type(e)}")
            raise e

    async def post(
            self, url, *,
            data: Any = None, json: Any = None, params: Any = None, timeout: Union[float, tuple] = None,
            cookies: Dict = None, headers: Dict = None, allow_redirects: bool = True,
            enable_proxy: Union[bool] = None, impersonate: str = None, verify: bool = True
    ):
        try:
            # 会话
            self.ensure_session()

            # 代理配置
            _enable_proxy = self.enable_proxy
            if enable_proxy is not None:
                _enable_proxy = enable_proxy
            if _enable_proxy and not self.proxy:
                await self.init_proxy(_enable_proxy)
            _proxy = self.proxy
            if not _enable_proxy:
                _proxy = None

            start = time.time()
            _impersonate = random.choice(self.enum_list)
            return await self.session.get(
                url=url, params=params, data=data, json=json,
                headers=headers, cookies=cookies, proxy=_proxy,
                allow_redirects=allow_redirects, timeout=timeout, impersonate=impersonate, verify=verify
            )
        except (ProxyGetFailException, RequestsError) as tme:
            LOGGER.info(f"超时[{self.proxy}]: {type(tme)}")
            raise RetryRequest(tme)
        except Exception as e:
            LOGGER.info(f"异常[{self.proxy}]: {type(e)}")
            raise e

    async def options(
            self, url, *,
            data: Any = None, json: Any = None, params: Any = None, timeout: Union[float, tuple] = None,
            cookies: Dict = None, headers: Dict = None, allow_redirects: bool = True,
            enable_proxy: Union[bool] = None, impersonate: str = None, verify: bool = True
    ):
        try:
            # 会话
            self.ensure_session()

            # 代理配置
            _enable_proxy = self.enable_proxy
            if enable_proxy is not None:
                _enable_proxy = enable_proxy
            if _enable_proxy and not self.proxy:
                await self.init_proxy(_enable_proxy)
            _proxy = self.proxy
            if not _enable_proxy:
                _proxy = None

            start = time.time()
            _impersonate = random.choice(self.enum_list)
            return await self.session.options(
                url=url, params=params, data=data, json=json,
                headers=headers, cookies=cookies, proxy=_proxy,
                allow_redirects=allow_redirects, timeout=timeout, impersonate=impersonate, verify=verify
            )
        except (ProxyGetFailException, RequestsError) as tme:
            LOGGER.info(f"超时[{self.proxy}]: {type(tme)}")
            raise RetryRequest(tme)
        except Exception as e:
            LOGGER.info(f"异常[{self.proxy}]: {type(e)}")
            raise e

    async def reset_proxy(self):
        if self.enable_proxy and self.proxy:
            _proxy = self.proxy
            try:
                self.proxy = None
                await proxy_client.remove_proxy(_proxy.replace(f'{self.proxy_protocol}://', ''))
            except Exception as e:
                LOGGER.info(f"代理移除异常[{_proxy}]: {type(e)}")

    # 初始化session
    def ensure_session(self, session: AsyncSession = None):
        if not self.session:
            self.session = session or AsyncSession(
                timeout=self.timeout,
                headers=self.headers,
                loop=asyncio.get_event_loop()
            )

    async def init_proxy(self, enable_proxy: bool):
        if enable_proxy and not self.proxy:
            _proxy = await proxy_client.get_proxy()
            if not _proxy:
                # 代理加载失败时抛出异常，终止请求（不允许无代理请求）
                raise ProxyGetFailException()
            self.proxy = f'{self.proxy_protocol}://{_proxy}'

    # 关闭client
    async def close(self):
        try:
            if self.session:
                await self.session.close()
        except Exception as e:
            LOGGER.error(f'会话关闭异常: {type(e)}')
        finally:
            self.session = None



from distutils.core import setup
from setuptools import find_packages

with open('README.rst', 'r', encoding="utf-8") as f:
    long_description = f.read()

# warning!
# for pymongo==4.7.3 and mongo version 7.0
# if you want to use old version, switch branch to mongo4

setup(name='mdp-core',
      version='0.0.2',
      description='mdp crawl core for zsodata',
      long_description=long_description,
      author='zsodata',
      author_email='team@zso.io',
      url='http://www.zsodata.com',
      install_requires=[
      ],
      python_requires='>=3.7',
      license='BSD License',
      packages=find_packages(),
      platforms=['all'],
      include_package_data=True
      )



# encoding: utf-8
import asyncio
import json
import random
import threading
import time
import traceback
import aiohttp
from abc import ABC, abstractmethod
from aiohttp import ClientConnectorError, ServerDisconnectedError
from curl_cffi import requests
from ..exception.exceptions import RetryRequest
from ..lib import cfg, logger, notify, settings

from ..utils.decator import singleton

LOGGER = logger.get('代理')


class Proxy(ABC):
    def __init__(self, plat_code: str = None, supplier_code: str = None):
        self.plat_code = plat_code
        self.supplier_code = supplier_code
        self.pool = dict()  #
        self.min_pool_size = 3
        self._instance_lock = threading.Lock()
        self._last_load_time = 0
        self.min_reload_period = 5
        self.proxy_amount_pre_load = 1
        self.pool_url = f'http://fast.proxy.zcbot.cn/api/proxy/get/jd.sku_stock_pc_v1/1?mode=FI_FO'

    async def get_dynamic_proxy(self):
        """获取代理"""
        key = f"{self.plat_code}_{self.supplier_code}"

        # 检查是否需要加载代理
        if key not in self.pool or len(self.pool[key]) < self.min_pool_size:
            await self.reload(plat_code=self.plat_code, supplier_code=self.supplier_code)

        return random.choice(self.pool[key]) if key in self.pool else None

    async def reset_dynamic_proxy(self):
        """重置代理"""
        key = f"{self.plat_code}_{self.supplier_code}"
        if key in self.pool:
            self.pool[key] = []
            LOGGER.info("重置代理池: %s", key)

    async def remove_dynamic_proxy(self, plat_code: str = None, supplier_code: str = None, proxy: str = None):
        """移除代理"""
        key = f"{plat_code}_{supplier_code}"
        if key in self.pool and proxy in self.pool[key]:
            self.pool[key].remove(proxy)
            LOGGER.info("移除代理: %s", proxy)

        # 代理数量不足，触发加载
        if key not in self.pool or len(self.pool[key]) < self.min_pool_size:
            await self.reload(plat_code=plat_code, supplier_code=supplier_code)

    async def reload(self, plat_code: str = None, supplier_code: str = None):
        """提取代理"""
        key = f"{plat_code}_{supplier_code}"
        self._instance_lock.acquire(timeout=30)

        # 如果代理数量充足，则无需加载
        if key in self.pool and len(self.pool[key]) >= self.min_pool_size:
            self._instance_lock.release()
            LOGGER.info("无需加载代理: %s", len(self.pool[key]))
            return

        # 防止提取代理频率过高
        if (int(time.time()) - self._last_load_time) <= self.min_reload_period:
            time.sleep(5)

        # 提取代理(json格式返回)
        response = requests.get(self.pool_url, timeout=5, headers={'accept': 'application/json'})
        if response.status_code == 200 and response.text:
            rows = json.loads(response.text).get('data', [])
            if key not in self.pool:
                self.pool[key] = []

            for row in rows:
                ip_port = row.strip()
                if ip_port:
                    self.pool[key].append(ip_port)
                else:
                    LOGGER.error("代理数据异常: %s", row)

            self._last_load_time = int(time.time())
            LOGGER.info("加载代理: current=%s, total=%s", rows, len(self.pool[key]))
        else:
            # 异常情况
            time.sleep(10)
            LOGGER.error("代理提取异常: %s", response.text)

        self._instance_lock.release()
        return len(self.pool.get(key, [])) > 0


@singleton
class StaticProxy(Proxy):
    """长效静态代理"""

    def __init__(self, plat_code: str = None, supplier_code: str = None):
        super().__init__(plat_code, supplier_code)
        self.plat_code = plat_code
        self.supplier_code = supplier_code
        self.static_proxy = None

    async def get_proxy(self, plat_code: str = None, supplier_code: str = None):
        """获取代理"""
        if not self.static_proxy:
            await self._fetch_static_proxy(plat_code, supplier_code)

        return self.static_proxy

    async def reset_proxy(self, plat_code: str = None, supplier_code: str = None, proxy: str = None):
        """重置代理"""
        self.static_proxy = None

    # TODO 这里要根据业务改造查询和解析方式
    async def _fetch_static_proxy(self, plat_code: str = None, supplier_code: str = None):
        """获取代理配置信息"""
        _plat_code = plat_code or self.plat_code
        _supplier_code = supplier_code or self.supplier_code

        try:
            # 接口请求参数
            params = {
                'plat_code': _plat_code,
                'supplier_code': _supplier_code
            }
            api = cfg.get('ACCOUNT_PROXY_FETCH_API', '')
            async with aiohttp.ClientSession() as session:
                async with session.post(api, json=params) as rs:
                    if rs.status == 200:
                        js_data = await rs.json() or {}
                        proxy = js_data.get("data", '')
                        if proxy:
                            LOGGER.info(f"获取静态代理成功: proxy={proxy}, plat_code={_plat_code}, supplier_code={_supplier_code}")
                            self.static_proxy = proxy
                    else:
                        LOGGER.info(f"获取静态代理失败:{rs.text}")
        except (asyncio.exceptions.TimeoutError, ClientConnectorError, ServerDisconnectedError) as tme:
            LOGGER.info(f"--> 提取超时: plat_code={_plat_code}, supplier_code={_supplier_code}, e={type(tme)}")
            raise RetryRequest(tme)
        except Exception as e:
            LOGGER.error(f"--> 提取异常: plat_code={_plat_code}, supplier_code={_supplier_code}, e={traceback.format_exc()}")
            raise e

    # TODO 这里要根据业务改造请求和解析方式
    async def _lock_static_proxy(self, plat_code: str = None, supplier_code: str = None, proxy: str = None):
        """禁用代理配置信息"""
        _plat_code = plat_code or self.plat_code
        _supplier_code = supplier_code or self.supplier_code

        try:
            # 接口请求参数
            params = {
                'plat_code': _plat_code,
                'supplier_code': _supplier_code
            }
            api = cfg.get('ACCOUNT_PROXY_LOCK_API', 'xxxx')
            async with aiohttp.ClientSession() as session:
                async with session.get(api, params=params) as rs:
                    if rs.status == 200:
                        js_data = await rs.json() or {}
                        proxy = js_data.get("data", '')
                        if proxy:
                            LOGGER.info(f"静态代理重置成功: proxy={proxy}, plat_code={_plat_code}, supplier_code={_supplier_code}")
                            self.static_proxy = proxy

                    LOGGER.info(f"获取静态代理失败:{rs.text}")
        except (asyncio.exceptions.TimeoutError, ClientConnectorError, ServerDisconnectedError) as tme:
            LOGGER.info(f"--> 提取超时: plat_code={_plat_code}, supplier_code={_supplier_code}, e={type(tme)}")
            raise RetryRequest(tme)
        except Exception as e:
            LOGGER.error(f"--> 提取异常: plat_code={_plat_code}, supplier_code={_supplier_code}, e={traceback.format_exc()}")
            raise e



# -*- coding: UTF-8 -*-
"""常用工具类"""
import math
import re
import random

import platform
import pprint
import typing
from typing import Any, Tuple, Optional

T = typing.TypeVar('T')

# global encoding
ENCODING = 'utf-8'


def as_int(o: Any) -> Tuple[int, Optional[Exception]]:
    """
    object to int
    :param o: object
    :return: int value, err
    """
    try:
        return int(o), None
    except Exception as e:
        return 0, e


def as_float(o: Any) -> Tuple[float, Optional[Exception]]:
    """
    object to float
    :param o: object
    :return: float value, err
    """
    try:
        return float(o), None
    except Exception as e:
        return 0.0, e


def is_win() -> bool:
    """
    is currently running on windows
    :return: if current system is of Windows family
    """
    return platform.system().lower().startswith('win')


def pfmt(o: Any, *args, **kwargs) -> str:
    """
    pretty format object
    :param o: object
    :return: object string
    """
    return pprint.pformat(o, *args, **kwargs)


def parse_number(s, err=0):
    """
    解析数字，如果解析失败返回默认值
    :param s:
    :param err:
    :return:
    """
    try:
        val = s
        if isinstance(s, str):
            s = s.replace('¥', '').replace('￥', '').replace(',', '').replace('元', '').strip()
            val = float(s)
        # 防止NaN值出现（如excel中）
        if math.isnan(val):
            return err
        return val
    except ValueError:
        return err
    except Exception:
        return err


def parse_int(s, err=0):
    """
    解析整型数字，如果解析失败返回默认值
    :param s:
    :param err:
    :return:
    """
    try:
        if isinstance(s, str):
            s = s.replace('¥', '').replace('￥', '').replace(',', '').replace('元', '').strip()
        return int(s)
    except ValueError:
        return err
    except TypeError:
        return err


def join_dict(kv):
    """
    将dict键值对拼接成字符串
    :param kv:
    :return:
    """
    s = ''
    for (k, v) in kv.items():
        s += k + ': ' + v + '\r\n'
    return s.strip()


def random_code(n=5, alpha=True):
    """
    生成验证码
    :param n:
    :param alpha:
    :return:
    """
    s = ''  # 创建字符串变量,存储生成的验证码
    for i in range(n):  # 通过for循环控制验证码位数
        num = random.randint(0, 9)  # 生成随机数字0-9
        if alpha:  # 需要字母验证码,不用传参,如果不需要字母的,关键字alpha=False
            upper_alpha = chr(random.randint(65, 90))
            lower_alpha = chr(random.randint(97, 122))
            num = random.choice([num, upper_alpha, lower_alpha])
        s = s + str(num)
    return s


def match_ip_port(txt):
    """
    解析IP和端口
    :param txt:
    :return:
    """
    # /frontBrands/getBrandsAndProductInfos.action?orderBy=normal&shopInfoId=1
    if txt:
        arr = re.findall(r'(?:(?:[0,1]?\d?\d|2[0-4]\d|25[0-5])\.){3}(?:[0,1]?\d?\d|2[0-4]\d|25[0-5]):\d{0,5}',
                         txt.strip())
        if len(arr):
            return arr[0].strip()
    return ''


def list_of_groups(init_list, childern_list_len):
    """
    列表分割成多个小列表
    :param init_list:
    :param childern_list_len:
    :return:
    """
    list_of_groups = zip(*(iter(init_list),) * childern_list_len)
    end_list = [list(i) for i in list_of_groups]
    count = len(init_list) % childern_list_len
    end_list.append(init_list[-count:]) if count != 0 else end_list
    return end_list


if __name__ == '__main__':
    # print(half_split_float_range(5, 8))
    # print(half_split_int_range(3, 6))
    # print(half_split_int_range(3, 5))
    # print(half_split_int_range(3, 4))
    # print(get_timestamp13())
    # print(format_timestamp(int(time.time() * 1000), fmt="%Y/%m/%d"))
    pass



# encoding: utf-8
import asyncio
import time
import ssl
import random
from asyncio import StreamReader
from typing import Any, Optional, Dict, Union
from aiohttp import TCPConnector, ClientSession, ClientConnectorError, ServerDisconnectedError, ClientTimeout, ClientResponse
from aiohttp.typedefs import JSONDecoder, DEFAULT_JSON_DECODER
from mdp_core.lib import logger
from mdp_core.exception.exceptions import ProxyGetFailException, RetryRequest
from mdp_core.web import proxy as proxy_client

LOGGER = logger.get('异步请求')

# 默认超时配置
DEFAULT_TIMEOUT = ClientTimeout(
    total=5,
    connect=5,
    sock_connect=1,
    sock_read=5
)


class SSLFactory:
    ciphers = ('ECDH+AESGCM:DH+AESGCM:ECDH+AES256:DH+AES256:ECDH+AES128:DH+AES:ECDH+HIGH:'
               'DH+HIGH:ECDH+3DES:DH+3DES:RSA+AESGCM:RSA+AES:RSA+HIGH:RSA+3DES').split(":")

    @staticmethod
    def gen() -> ssl.SSLContext:
        random.shuffle(SSLFactory.ciphers)
        ciphers = ":".join(SSLFactory.ciphers)
        ciphers = ciphers + ":!aNULL:!eNULL:!MD5"
        context = ssl.create_default_context()
        context.set_ciphers(ciphers)
        return context


class HttpClientResponse:

    def __init__(self, response: ClientResponse, seconds_cost: float = None):
        self.response = response
        self._seconds_cost = seconds_cost

    @property
    def url(self) -> str:
        return str(self.response.url)

    @property
    def status_code(self) -> int:
        return self.response.status

    @property
    def cookies(self) -> Dict:
        return self.response.cookies

    @property
    def content(self) -> StreamReader:
        return self.response._body

    @property
    def text(self, default: Any = None, encoding: Optional[str] = None, errors: str = "strict") -> str:
        if self.response._body:
            if encoding is None:
                encoding = self.response.get_encoding()
            return self.response._body.decode(  # type: ignore[no-any-return,union-attr]
                encoding, errors=errors
            )

        return default or ''

    def json(self, default: Any = None, *, encoding: Optional[str] = None, loads: JSONDecoder = DEFAULT_JSON_DECODER) -> Any:
        if self.response._body:
            stripped = self.response._body.strip()  # type: ignore[union-attr]
            if not stripped:
                return None
            if encoding is None:
                encoding = self.response.get_encoding()
            return loads(stripped.decode(encoding))

        return default or {}

    def total_seconds(self) -> float:
        return self._seconds_cost or -1


class HttpClient:

    def __init__(
            self,
            timeout: Union[float, ClientTimeout] = None,
            headers=None,
            ssl_context: ssl.SSLContext = None,
            allow_redirects: bool = True,
            enable_proxy: bool = True,
            proxy_protocol: str = 'http',
            session: ClientSession = None
    ):
        self.timeout = self._timeout(timeout)
        self.headers = headers
        self.ssl_context = ssl_context or SSLFactory.gen()
        self.allow_redirects = allow_redirects

        # 全局启用代理
        self.enable_proxy = enable_proxy
        self.proxy = None
        self.proxy_protocol = proxy_protocol or 'http'

        # 会话
        self.session = None
        self.ensure_session(session)

    async def get(
            self, url, *,
            params: Any = None, timeout: Union[float, ClientTimeout] = None,
            cookies: Dict = None, headers: Dict = None, allow_redirects: bool = True,
            enable_proxy: Union[bool] = None
    ) -> HttpClientResponse:
        try:
            # 会话verify
            self.ensure_session()

            # 代理配置
            _enable_proxy = self.enable_proxy and enable_proxy
            if _enable_proxy and not self.proxy:
                await self.init_proxy(_enable_proxy)
            _proxy = self.proxy
            if not _enable_proxy:
                _proxy = None

            start = time.time()
            async with self.session.get(
                    url=url, params=params,
                    headers=headers, cookies=cookies, proxy=_proxy,
                    allow_redirects=allow_redirects, timeout=self._timeout(timeout)
            ) as resp:
                await resp.read()
                return HttpClientResponse(response=resp, seconds_cost=(time.time() - start))
        except (ProxyGetFailException, asyncio.exceptions.TimeoutError, ClientConnectorError, ServerDisconnectedError) as tme:
            LOGGER.info(f"超时[{self.proxy}]: {type(tme)}")
            raise RetryRequest(tme)
        except Exception as e:
            LOGGER.info(f"异常[{self.proxy}]: {type(e)}")
            raise e

    async def post(
            self, url, *,
            data: Any = None, json: Any = None, params: Any = None, timeout: Union[float, tuple, ClientTimeout] = None,
            cookies: Dict = None, headers: Dict = None, allow_redirects: bool = True,
            enable_proxy: Union[bool] = None
    ) -> HttpClientResponse:
        try:
            # 会话
            self.ensure_session()

            # 代理配置
            _enable_proxy = self.enable_proxy and enable_proxy
            if _enable_proxy and not self.proxy:
                await self.init_proxy(_enable_proxy)
            _proxy = self.proxy
            if not _enable_proxy:
                _proxy = None

            start = time.time()
            async with self.session.post(
                    url=url, data=data, json=json, params=params,
                    headers=headers, cookies=cookies, proxy=_proxy,
                    allow_redirects=allow_redirects, timeout=timeout
            ) as resp:
                await resp.read()
                return HttpClientResponse(response=resp, seconds_cost=(time.time() - start))
        except (ProxyGetFailException, asyncio.exceptions.TimeoutError, ClientConnectorError, ServerDisconnectedError) as tme:
            LOGGER.info(f"超时[{self.proxy}]: {type(tme)}")
            raise RetryRequest(tme)
        except Exception as e:
            LOGGER.info(f"异常[{self.proxy}]: {type(e)}")
            raise e

    async def init_proxy(self, enable_proxy: bool):
        if enable_proxy and not self.proxy:
            _proxy = await proxy_client.get_proxy()
            if not _proxy:
                # 代理加载失败时抛出异常，终止请求（不允许无代理请求）
                raise ProxyGetFailException()
            self.proxy = f'{self.proxy_protocol}://{_proxy}'

    # 重置代理
    async def reset_proxy(self):
        if self.enable_proxy and self.proxy:
            _proxy = self.proxy
            try:
                self.proxy = None
                await proxy_client.remove_proxy(_proxy.replace(f'{self.proxy_protocol}://', ''))
            except Exception as e:
                LOGGER.info(f"代理移除异常[{_proxy}]: {type(e)}")

    # 超时时间构造
    def _timeout(self, timeout: Union[float, tuple, ClientTimeout]):
        if isinstance(timeout, ClientTimeout):
            return timeout
        elif isinstance(timeout, float):
            return ClientTimeout(total=timeout)
        else:
            return DEFAULT_TIMEOUT

    # 初始化session
    def ensure_session(self, session: ClientSession = None):
        if not self.session or self.session.closed:
            self.session = session or ClientSession(
                timeout=self.timeout,
                headers=self.headers,
                connector=TCPConnector(ssl=self.ssl_context)
            )

    # 关闭client
    async def close(self):
        try:
            if self.session:
                await self.session.close()
        except Exception as e:
            LOGGER.error(f'会话关闭异常: {type(e)}')
        finally:
            self.session = None
